@@ -123,6 +123,40 @@ static void nvidia_pci_enable_intx(device_t dev)
     os_pci_write_word(dev, PCIR_COMMAND, word);
 }
 
+void nvidia_pci_save_config_space(
+    nv_stack_t *sp,
+    device_t dev
+)
+{
+    struct nvidia_softc *sc = device_get_softc(dev);
+    nv_state_t *nv = sc->nv_state;
+    NvS16 i;
+
+    for (i = 0; i < NVRM_PCICFG_NUM_DWORDS; i++)
+    {
+        os_pci_read_dword(dev, (i << 2), &nv->pci_cfg_space[i]);
+    }
+}
+
+void nvidia_pci_restore_config_space(
+    nv_stack_t *sp,
+    device_t dev
+)
+{
+    struct nvidia_softc *sc = device_get_softc(dev);
+    nv_state_t *nv = sc->nv_state;
+    NvS16 i;
+    NvU32 dword;
+
+    for (i = NVRM_PCICFG_NUM_DWORDS - 1; i >= 0; i--)
+    {
+        os_pci_read_dword(dev, (i << 2), &dword);
+        if (dword != nv->pci_cfg_space[i]) {
+            os_pci_write_dword(dev, (i << 2), nv->pci_cfg_space[i]);
+        }
+    }
+}
+
 void nvidia_pci_check_config_space(
     nv_stack_t *sp,
     device_t dev,
